#!/usr/bin/env python
# Class architecture (C) 2005 manatlan
# Code and XML pages (C) 2005 Alexis ROBERT

"""
This class pilot remotly a VLC with his HTTP server
and special pages.
"""

import os,urllib

from xmldoc import XmlDoc


def int2bool(arg):
    if int(arg) == 0:
        return False
    else:
        return True

def urlread(iurl):
    f = urllib.urlopen(iurl)
    data = f.read()
    f.close()   
    return data
    
class PlaylistItem:
    "Item for VLC playlist"
    def __init__(self, id, title, current = 0):
        self.item = {"id": int(id), "title": title, "current": int2bool(current)}

    def __getitem__(self, val):
        return self.item[val]

    def __setitem__(self, val, content):
        self.item[val] = content

class VLCError:
    "Standard msg error (raise str is deprecated)"
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg

class PilotVlc:
    port = 8081
    
    exe = "/usr/bin/vlc"
    ipFbx="freeplayer.freebox.fr"
    
    @staticmethod
    def start(file=""):
        cmd=[
            PilotVlc.exe,
                "--intf", "rc",
                "--extraintf","http",
                "--http-src","common/vlc-html",
                "--http-host",":%d" % PilotVlc.port,
                """--sout="#std" """,
                "--sout-standard-access=udp",
                "--sout-standard-mux=ts",
                "--sout-standard-url=%s:1234" % PilotVlc.ipFbx,
                "--sout-ts-pid-video=68",
                "--sout-ts-pid-audio=69",
                "--sout-ts-pcr=80",
                "--sout-ts-dts-delay=400",
                #~ "--color",
                #~ "--play-and-stop",
                "--sout=#transcode:std",
                "--sout-transcode-ab=384",
                "--sout-transcode-acodec=mpga",
                "--sout-transcode-channels=2",
                "--sout-transcode-vb=9000",
                "--sout-transcode-vcodec=mp2v",
                
                file
                ]
        print " ".join(cmd)
        os.spawnvp(os.P_NOWAIT,cmd[0],cmd)
        
    def __init__(self, ipVlc="127.0.0.1"):
        self.ipVlc = ipVlc
        #if not self.check():
        #    raise VLCError("VLC with templates is not detected at http://%s:%s" % (self.ipVlc, self.port))

    def __getitem__(self, var):
        url = "http://%s:%s/value.htm?name=%s" % (self.ipVlc, self.port, var)
        dom = XmlDoc(url=url)
        ln=dom.getAll("content")
        return ln[0].getText()
    
    def check(self):
        "Checks if this host uses VLC with our XML files"
        try:
            url = "http://%s:%s/check.htm" % (self.ipVlc, self.port, var)
            dom = XmlDoc(url=url)
            ln=dom.getAll("check")            
            if ln[0].getAttr("ok") == "1":
                return True
            else:
                return False
        except:
            return False

    def getVersion(self, extended = False):
        "Get the VLC version"
        if extended:
            return self["copyright"]
        else:
            return self["version"]

    def isPlaying(self):
        if self["stream_state"] == "playing":
            return True
        else:
            return False

    def getPlaylist(self):
        dest = []
        data=urlread("http://%s:%s/playlist.htm" % (self.ipVlc, self.port)).replace("&","&amp;")
        
        doc = XmlDoc(xml=data)
        for item in doc.getAll("item"):
            dest.append(PlaylistItem(item.getAttr("id"), item.getText(), item.getAttr("current")))

        return dest

    def getCurrentItem(self):
        data=urlread("http://%s:%s/playlist.htm" % (self.ipVlc, self.port)).replace("&","&amp;")
        
        doc = XmlDoc(xml=data)
        for item in doc.getAll("item"):
            if item.getAttr("current") == "1":
                return PlaylistItem(item.getAttr("id"), item.getText(), 1)

        return None

    def getTitle(self):
        try:
            a=self.getCurrentItem()
            return a["title"]
        except:
            return "unknow name"

    def getLength(self):
        return float(self["stream_length"])

    def getTime(self):
        return float(self["stream_time"])

    def getPercent(self):
        length = self.getLength()
        time = self.getTime()
        if not length == 0:
            return int((time / length) * 100)
        else:
            return 0

    def getVolume(self):
        return int(self["volume"])

    def setVolume(self, v):
        if v < 0 or v > 486:
            raise VLCError("Volume must be < 0 or > 486")
        
        urllib.urlopen("http://%s:%s/control.htm?control=volume&value=\"%s\"" % (self.ipVlc, self.port, str(v)))

    def add(self, file, sout = None):
        if sout:
            extraurl = "&sout=\"%s\"" % sout
        else: extraurl = ""
        urllib.urlopen("http://%s:%s/control.htm?control=add&mrl=\"%s\"%s" % (self.ipVlc, self.port, file, extraurl))

    def play(self, item = None):
        if self["stream_state"] == "paused":
            self.pause()
        else:
            if item==None:
                currentitem = self.getCurrentItem()
                if not currentitem:
                    currentitem = self.getPlaylist()[0]
                item = currentitem["id"]
                
            urllib.urlopen("http://%s:%s/control.htm?control=play&item=%s" % (self.ipVlc, self.port, str(item)))

    def pause(self):
        urllib.urlopen("http://%s:%s/control.htm?control=pause" % (self.ipVlc, self.port))

    def stop(self):
        urllib.urlopen("http://%s:%s/control.htm?control=stop" % (self.ipVlc, self.port))

    def next(self):
        urllib.urlopen("http://%s:%s/control.htm?control=next" % (self.ipVlc, self.port))

        if self.isPlaying():
            self.play()

    def prev(self):
        urllib.urlopen("http://%s:%s/control.htm?control=previous" % (self.ipVlc, self.port))

        if self.isPlaying():
            self.play()

    def seek(self, seek_value):
        urllib.urlopen("http://%s:%s/control.htm?control=seek&seek_value=\"%s\"" % (self.ipVlc, self.port, seek_value))

    def shutdown(self):
        urllib.urlopen("http://%s:%s/control.htm?control=shutdown" % (self.ipVlc, self.port))
       
    def empty(self):
        urllib.urlopen("http://%s:%s/control.htm?control=empty" % (self.ipVlc, self.port))
